
// - # FGCargoMonitor

/*!
 *  \details   Simple cargo monitor subscribing system
 *  \author    Kiss Ferenc
 *  \version   1.0
 *  \date      2013
 *  \copyright GNU Public License.
 *  \note      See \ref Example for simple and full example
 */

require("FGMonitoredGoal.nut");
require("FGCompanyCargos.nut");
require("FGTownCargos.nut");
require("FGIndustryCargos.nut");

class FGCargoMonitor { // valami
	/*! \page REFRESH_TYPES REFRESH_TYPES documentation page
	 This type use in \link FGCargoMonitor::SubscribeToMonitor \endlink refresh_interval variable.
	 \section REFRESH_INVALID
	 Invalid refresh type.
	 \section REFRESH_IMMEDIATELY
	 Send update immediately to delegate SendCargoUpdate(company_id, goal_id, transported) function.
	 \section REFRESH_EVERY_MONTHS
	 Send update first day of every montsh to delegate SendCargoUpdate(company_id, goal_id, transported) function.
	 Invalid refresh type.
	 \section REFRESH_EVERY_QARTER_YEARS
	 Send update first day of every quarter years (accurately first day of next months after quarter year: 1. month first day, 4. month first day, 7. month first day...) to delegate SendCargoUpdate(company_id, goal_id, transported) function.
	 Invalid refresh type.
	 \section REFRESH_EVERY_HALF_YEARS
	 Send update first day of every half years (accurately first day of next months after quarter months: 1. month first day, 7. month first day...) to delegate SendCargoUpdate(company_id, goal_id, transported) function.
	 Invalid refresh type.
	 \section REFRESH_EVERY_YEARS
	 Send update first day of every years (accurately first day of next months after quarter months: 1. month first day...) to delegate SendCargoUpdate(company_id, goal_id, transported) function.
	 Invalid refresh type.
	 */
	
	
	/*! \page MONITOR_TYPES MONITOR_TYPES documentation page
	 This type use in \link FGCargoMonitor::SubscribeToMonitor \endlink monitor_type variable.
	 \section MONITOR_INVALID
	 Invaid monitor type.
	 \section MONITOR_ALL
	 Subscribe to all delivery and pickup messages.
	 \section MONITOR_ALL_DELIVERY
	 Subscribe to all delivery messages.
	 \section MONITOR_ALL_PICKUP
	 Subscribe to all pickup messages.
	 \section MONITOR_TOWN_DELIVERY
	 Subscribe to only one town delivery messages.
	 \section MONITOR_TOWN_PICKUP
	 Subscribe to only one town  pickup messages.
	 \section MONITOR_INDUSTRY_DELIVERY
	 Subscribe to only one industry delivery messages.
	 \section MONITOR_INDUSTRY_PICKUP
	 Subscribe to only one industry pickup messages.
	 */
	
	
	
	/*! \page LOGLLVEL_TYPES LOGLLVEL_TYPES documentation page
	 You can create an instance whit these types in this class contructor: cargoMonitor = FGCargoMonitor(delegate, loglevel);
	 \section LOGLEVEL_OFF
	 Disable logs.
	 \section LOGLEVEL_ERRORS_ONLY
	 Logging error messages only.
	 \section LOGLEVEL_ERRORS_AND_WARNINGS
	 Logging error and warning messages.
	 \section LOGLEVEL_ALL
	 Logging all messages. This usually for Debug.
	 */
	
	// REFRESH_TYPES
	
	/*!
	 * Invalid refresh type.
	 */
	static REFRESH_INVALID = 0;
	
	/*!
	 * Send update immediately to delegate SendCargoUpdate(company_id, goal_id, transported) function.
	 */
	static REFRESH_IMMEDIATELY = 1;
	
	/*!
	 * Send update first day of every montsh to delegate SendCargoUpdate(company_id, goal_id, transported) function.
	 */
	static REFRESH_EVERY_MONTHS = 2;
	
	/*!
	 * Send update first day of every quarter years (accurately first day of next months after quarter year: 1. month first day, 4. month first day, 7. month first day...) to delegate SendCargoUpdate(company_id, goal_id, transported) function.
	 */
	static REFRESH_EVERY_QARTER_YEARS = 3;
	
	/*!
	 * Send update first day of every half years (accurately first day of next months after quarter months: 1. month first day, 7. month first day...) to delegate SendCargoUpdate(company_id, goal_id, transported) function.
	 */
	static REFRESH_EVERY_HALF_YEARS = 4;
	
	/*!
	 * Send update first day of every years (accurately first day of next months after quarter months: 1. month first day...) to delegate SendCargoUpdate(company_id, goal_id, transported) function.
	 */
	static REFRESH_EVERY_YEARS = 5;
	
	
	
	// MONITOR_TYPES
	/*!
	 * Invaid monitor type.
	 */
	static MONITOR_INVALID = 0;
	
	/*!
	 * Subscribe to all delivery and pickup messages.
	 */
	static MONITOR_ALL = 1;
	
	/*!
	 * Subscribe to all delivery messages.
	 */
	static MONITOR_ALL_DELIVERY = 2;
	
	/*!
	 * Subscribe to all pickup messages.
	 */
	static MONITOR_ALL_PICKUP = 3;
	
	/*!
	 * Subscribe to only one town delivery messages
	 */
	static MONITOR_TOWN_DELIVERY = 4;
	
	/*!
	 * Subscribe to only one town pickup message.
	 */
	static MONITOR_TOWN_PICKUP = 5;
	
	/*!
	 * Subscribe to only one industry delivery messages
	 */
	static MONITOR_INDUSTRY_DELIVERY = 6;
	
	/*!
	 * Subscribe to only one industry pickup messages
	 */
	static MONITOR_INDUSTRY_PICKUP = 7;
	
	// LOGLLVEL_TYPES
	/*!
	 * Disable logs.
	 */
	static LOGLEVEL_OFF = 0;
	
	/*!
	 * Logging error messages only.
	 */
	static LOGLEVEL_ERRORS_ONLY = 1;
	
	/*!
	 * Logging error and warning messages.
	 */
	static LOGLEVEL_ERRORS_AND_WARNINGS = 2;
	
	/*!
	 * Logging all messages. This usually for Debug.
	 */
	static LOGLEVEL_ALL = 3;
	
	
	monitor_delegate = null; // a delegate az excepted IDENTIFIER volt, azert kellett atirni
	subscribers = null; // ezek iratkoztak fel a frissitesi uzenetekre
	cached_subscribers = null; // ebben fogom tarolni az egy nap alatt erkezo uj feliratkozokat, akiket csak az Update() vegen fogok hozzaadni
	companies = null;
	
	logLevel = 0; // 0: nothing, 1: error only, 2: error and warning, 3: all
	startyear = 0;
	
	need_before_month = false; // ezt nem kell menteni, ez csak a belso lekerdezes miatt van, mert a belso lekerdezes a honap elso nap elejen kerdezodik le, akkor meg meg az elozo honapra vagyunk kivancsiak
	
	log_date_enabled = true; // kell-e date a log kiiratasba
	
	constructor (a_delegate, a_logLevel) {
		this.monitor_delegate = a_delegate;
		this.logLevel = a_logLevel;
		this.startyear = GSDate.GetYear(GSDate.GetCurrentDate());
		
		this.subscribers = [];
		this.cached_subscribers = [];
		this.companies = [];
	}
	
	/*!
	 *	Simple save all data with this function
	 *
	 *	\return table		table with all monitoring data
	 */
	function SaveData();
	
	/*!
	 *	Load data from save data
	 *
	 *	\param loaded_data			Contain the save data
	 *
	 *	\pre							loaded_data != null
	 */
	function LoadFromData(loaded_data);
	
	/*!
	 *	You need to run this every day
	 */
	function Update();
	
	/*!
	 *	You can subscribe to monitoring with this function
	 *
	 *	\param company_id			Subscribing to company cargo changes
	 *	\param cargo_id			Subscribing to specific cargo
	 *	\param goal_id			This is the individual ID, that represent the goal
	 *	\param refresh_interval	\ref REFRESH_TYPES, The update frequency
	 *	\param monitor_type		\ref MONITOR_TYPES, What updates need for my script
	 *	\param type_id			Only need this variable valid, when you want specific MONITOR_TYPES subscribing
	 *
	 *	\return boolean			Return true if the all parameter is valid
	 *
	 *	\pre						GSCompany.ResolveCompanyID(company_id) != GSCompany.COMPANY_INVALID;
	 *	\pre						GSCargo.IsValidCargo(cargo_id)
	 *	\pre						goal_id != null
	 *	\pre						\link FGCargoMonitor::IsValidRefreshInterval(refresh_interval) \endlink
	 *	\pre						if monitor_type ==  \ref MONITOR_TOWN_DELIVERY || monitor_type == \ref MONITOR_TOWN_PICKUP) then GSTown.IsValidTown(type_id)
	 *	\pre						if monitor_type ==  \ref MONITOR_INDUSTRY_DELIVERY || monitor_type == \ref MONITOR_INDUSTRY_PICKUP) then GSIndustry.IsValidIndustry(type_id)
	 */
	function SubscribeToMonitor(company_id, cargo_id, goal_id, refresh_interval, monitor_type, type_id);
	
	/*!
	 *	Unsubscribe from specific company and specific goal
	 *
	 *	\param company_id			Subscribing to company cargo changes
	 *	\param goal_id			This is the individual ID, that represent the goal
	 *
	 *	\pre						company_id != null
	 *	\pre						goal_id != null
	 */
	function UnSubscribeFromMonitor(company_id, goal_id);
	
	/*!
	 *	Unsubscribe from all monitoring from specific company
	 *
	 *	\param company_id		Subscribing to company cargo changes
	 *
	 *	\pre						company_id != null
	 */
	function UnSubscribeCompanyFromMonitor(company_id);
	
	/*!
	 *	Unsubscribe from all monitoring (recommended at end of game)
	 */
	function UnSubscribeAllFromMonitor();
	
	/*!
	 *	Is refresh_type valid?
	 *
	 *	\param refresh_interval	\ref REFRESH_TYPES
	 *
	 *	\return bool		the refresh_interval is valid
	 */
	function IsValidRefreshInterval(refresh_interval);
	
	/*!
	 *	Query the specific company: how much specific cargo delivered in the last months
	 *
	 *	\note					if the last_months == 0 then return \link FGCargoMonitor::AmountOfAllDeliveredCargo(company_id, cargo_id) \endlink<br>
	 *							if the last_months == 1 then return the actual month transported cargo_id<br>
	 *							if the last_months > 1 then return the last_months transported cargo_id (included actual month)
	 *
	 *	\param company_id			Query this specific company
	 *	\param cargo_id			Query this specific cargo type
	 *	\param last_months		How much last months transported cargo need
	 *
	 *	\pre						company_id != null
	 *	\pre						cargo_id != null
	 */
	function AmountOfLastMonthDeliveredCargo(company_id, cargo_id, last_months);
	
	/*!
	 *	Query the specific company: how much specific cargo delivered form subscrubing
	 *
	 *	\param company_id			Query this specific company
	 *	\param cargo_id			Query this specific cargo type
	 *
	 *	\pre						company_id != null
	 *	\pre						cargo_id != null
	 */
	function AmountOfAllDeliveredCargo(company_id, cargo_id);
	
	/*!
	 *	Query the specific company: how much specific cargo picked up in the last months
	 *
	 *	\note					if the last_months == 0 then return \link FGCargoMonitor::AmountOfAllPickedupCargo(company_id, cargo_id) \endlink<br>
	 *							if the last_months == 1 then return the actual month transported cargo_id<br>
	 *							if the last_months > 1 then return the last_months transported cargo_id (included actual month)
	 *
	 *	\param company_id			Query this specific company
	 *	\param cargo_id			Query this specific cargo type
	 *	\param last_months		How much last months transported cargo need
	 *
	 *	\pre						company_id != null
	 *	\pre						cargo_id != null
	 */
	function AmountOfLastMonthPickedupCargo(company_id, cargo_id, last_months);
	
	/*!
	 *	Query the specific company: how much specific cargo picked up form subscrubing
	 *
	 *	\param company_id			Query this specific company
	 *	\param cargo_id			Query this specific cargo type
	 *
	 *	\pre						company_id != null
	 *	\pre						cargo_id != null
	 */
	function AmountOfAllPickedupCargo(company_id, cargo_id);
	
	/*!
	 *	Query the specific company: how much specific goal delivered or picked up in the last months
	 *
	 *	\note					if the last_months == 0 then return \link FGCargoMonitor::AmountOfAllGoal(company_id, goal_id) \endlink<br>
	 *							if the last_months == 1 then return the actual month transported cargo of goal_id<br>
	 *							if the last_months > 1 then return the last_months transported cargo of goal_id (included actual month)
	 *
	 *	\param company_id			Query this specific company
	 *	\param goal_id			Query this specific goal of company_id
	 *	\param last_months		How much last months transported cargo need
	 *
	 *	\pre						company_id != null
	 *	\pre						goal_id != null
	 */
	function AmountOfLastMonthGoal(company_id, goal_id, last_months);
	
	/*!
	 *	Query the specific company: how much specific goal delivered or picked up form subscrubing
	 *
	 *	\param company_id			Query this specific company
	 *	\param goal_id			Query this specific goal of company_id
	 *
	 *	\pre						company_id != null
	 *	\pre						goal_id != null
	 */
	function AmountOfAllGoal(company_id, goal_id);
	
	/*!
	 *	This function remove "[year. month. day.]" from log string.
	 *
	 */
	function DisableGameDateFromLog();
}

function FGCargoMonitor::SaveData() {
	local a_subscribers = [];
	foreach (subscriber in this.subscribers) {
		a_subscribers.append(subscriber.SaveData());
	}
	
	local a_cached_subscribers = [];
	foreach (subscriber in this.cached_subscribers) {
		a_cached_subscribers.append(subscriber.SaveData());
	}
	
	local a_companies = [];
	foreach (company in this.companies) {
		a_companies.append(company.SaveData());
	}
	
	return {
		startyear = this.startyear,
		
		subscribers = a_subscribers,
		cached_subscribers = a_cached_subscribers, // ezt is elmentjuk a biztonsag kedveert
		companies = a_companies,
	}
}

function FGCargoMonitor::LoadFromData(loaded_data) {
	this.startyear = loaded_data.startyear;
	
	// subscriber eseten nem kell a LoadFromData, mert automatikusan ugy hozom letre
	foreach (subscriber_list in loaded_data.subscribers) {
		local subscriber = FGMonitoredGoal(subscriber_list);
		this.subscribers.append(subscriber);
	}
	
	foreach (subscriber_list in loaded_data.cached_subscribers) {
		local subscriber = FGMonitoredGoal(subscriber_list);
		this.cached_subscribers.append(subscriber);
	}
	
	foreach (company_list in loaded_data.companies) {
		local company = FGCompanyCargos(company_list.company_id, this.logLevel);
		company.LoadFromData(company_list);
		this.companies.append(company);
	}
}

function FGCargoMonitor::GetLogGameDateString() {
	if (!this.log_date_enabled)
		return "";

	local date = GSDate.GetCurrentDate();
	local month = GSDate.GetMonth(date);
	local day = GSDate.GetDayOfMonth(date);
	
	return "[" + GSDate.GetYear(date) + ". " + (month < 10 ? ("0" + month) : month) + ". " + (day < 10 ? ("0" + day) : day) + ".]";
}

function FGCargoMonitor::DisableGameDateFromLog() {
	this.log_date_enabled = false;
}

function FGCargoMonitor::Update() {
	// megnezzuk, hogy van-e olyan feliratkozom aki meg nem szerepel a listankban, es akkor hozzaadjuk
	foreach (subscriber in this.subscribers) {
		local van = false;
		foreach (company in this.companies) {
			if (company.company_id == subscriber.company_id)
				van = true;
		}
		
		if (!van) {
			local newCompany = FGCompanyCargos(subscriber.company_id, this.logLevel);
			this.companies.append(newCompany);
		}
	}
	
	local year = GSDate.GetYear(GSDate.GetCurrentDate());
	local elteltMonths = ((year - this.startyear) * 12) + GSDate.GetMonth(GSDate.GetCurrentDate()) - 1; // azert a -1, hogy 0 legyen a kezdohonap a lista miatt
	
	foreach (company in this.companies) {
		// elso alkalommal frissitjuk a honapokat, es hozzaadjuk az ujat
		company.UpdateAllMonths(elteltMonths);
		
		// osszegyujtjuk a cargokat, ami all monitorozasuak
		local alredyMonitoredDeliveryTown = [];
		local alredyMonitoredPickupTown = [];
		local alredyMonitoredDeliveryIndustry = [];
		local alredyMonitoredPickupIndustry = [];
		
		// csak ket kerest csinalunk, mert eloszor tudnunk kell az osszes dologra feliratkozokat, mert ezen belul is ertesiteni fogjuk az egyeni feliratkozokat
		foreach (subscriber in this.subscribers) {
			if (company.company_id != subscriber.company_id)
				continue;
			
			// all eseteben eleg csak az osszes varostol lekerdezni
			
			local needMonitoringAllDelivery = false;
			local needMonitoringAllPickup = false;
			
			switch (subscriber.monitor_type) {
				case FGCargoMonitor.MONITOR_ALL:
					needMonitoringAllDelivery = true;
					needMonitoringAllPickup = true;
					break;
				case FGCargoMonitor.MONITOR_ALL_DELIVERY:
					needMonitoringAllDelivery = true;
					break;
				case FGCargoMonitor.MONITOR_ALL_PICKUP:
					needMonitoringAllPickup = true;
					break;
			}
			
			if (needMonitoringAllDelivery) {
				local van = false;
				foreach (cargo_id in alredyMonitoredDeliveryTown) {
					if (cargo_id == subscriber.cargo_id) {
						van = true;
						break;
					}
				}
				
				if (van)
					continue;
				
				// osszes varos osszes delivery-t vesszuk
				local towns = GSTownList();
				foreach (town_id, _ in towns) {
					if (!GSTown.IsValidTown(town_id)) {
						if (this.logLevel >= 2)
							GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::Update] (Delivery All) Invalid town id: " + town_id);
						continue;
					}
					
					local towntransported = GSCargoMonitor.GetTownDeliveryAmount(company.company_id, subscriber.cargo_id, town_id, true);
					if (towntransported > 0) {
						company.AddDeliveryTownCargo(subscriber.cargo_id, towntransported, elteltMonths, town_id);
						
						// towntransportedet el kell kuldeni
						// elvileg itt azt kuldjuk el, aki konkretan egy varos beerkezesere iratkozott fel
						foreach (asubscriber in this.subscribers) {
							local monitortypeok = false;
							if (asubscriber.monitor_type == FGCargoMonitor.MONITOR_ALL ||
								asubscriber.monitor_type == FGCargoMonitor.MONITOR_ALL_DELIVERY ||
								(asubscriber.monitor_type == FGCargoMonitor.MONITOR_TOWN_DELIVERY && asubscriber.type_id == town_id))
								monitortypeok = true;
							
							if (monitortypeok &&
								asubscriber.company_id == company.company_id &&
								asubscriber.cargo_id == subscriber.cargo_id &&
								asubscriber.refresh_interval == FGCargoMonitor.REFRESH_IMMEDIATELY) {
									this.monitor_delegate.SendCargoUpdate(asubscriber.company_id, asubscriber.goal_id, towntransported);
							}
						}
					}
					
					alredyMonitoredDeliveryTown.append(subscriber.cargo_id);
				}
			}
			
			if (needMonitoringAllPickup) {
				local van = false;
				foreach (cargo_id in alredyMonitoredPickupTown) {
					if (cargo_id == subscriber.cargo_id) {
						van = true;
						break;
					}
				}
				
				if (!van) {
					// osszes varos osszes delivery-t vesszuk
					local towns = GSTownList();
					foreach (town_id, _ in towns) {
						if (!GSTown.IsValidTown(town_id)) {
							if (this.logLevel >= 2)
								GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::Update] (Pickup All) Invalid town id: " + town_id);
							continue;
						}
						
					
						local towntransported = GSCargoMonitor.GetTownPickupAmount(subscriber.company_id, subscriber.cargo_id, town_id, true);
						
						if (towntransported > 0) {
							company.AddPickupTownCargo(subscriber.cargo_id, towntransported, elteltMonths, town_id);
							
							// towntransportedet el kell kuldeni
							// elvileg itt azt kuldjuk el, aki konkretan egy varos beerkezesere iratkozott fel
							foreach (asubscriber in this.subscribers) {
								local monitortypeok = false;
								if (asubscriber.monitor_type == FGCargoMonitor.MONITOR_ALL ||
									asubscriber.monitor_type == FGCargoMonitor.MONITOR_ALL_PICKUP ||
									(asubscriber.monitor_type == FGCargoMonitor.MONITOR_TOWN_PICKUP && asubscriber.type_id == town_id))
									monitortypeok = true;
								
								if (monitortypeok &&
									asubscriber.company_id == company.company_id &&
									asubscriber.cargo_id == subscriber.cargo_id &&
									asubscriber.refresh_interval == FGCargoMonitor.REFRESH_IMMEDIATELY) {
										this.monitor_delegate.SendCargoUpdate(asubscriber.company_id, asubscriber.goal_id, towntransported);
								}
							}
						}
						
						alredyMonitoredPickupTown.append(subscriber.cargo_id);
					}
				}
				
				{
					// TODO: ezt a reszt kell kivenni, ha a nightlyban javitjak a hibat,
					// pontosabban le kell kerdezni a verzioszamot, ahonnan mar jo, es ha a feletti,
					// akkor nem kell ezt a reszt futtatni, tehat a regi verzio tamogatas miatt is kell
					
					// ebben az esetben az industy-t is ellenorizni kell
					van = false;
					foreach (cargo_id in alredyMonitoredPickupIndustry) {
						if (cargo_id == subscriber.cargo_id) {
							van = true;
							break;
						}
					}
					
					if (!van) {
						// osszes varos osszes delivery-t vesszuk
						local industries = GSIndustryList();
						foreach (industry_id, _ in industries) {
							if (!GSIndustry.IsValidIndustry(industry_id)) {
								if (this.logLevel >= 2)
									GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::Update] (Pickup All Industry) Invalid industry id: " + industry_id);
								continue;
							}
							
							local industrytransported = GSCargoMonitor.GetIndustryPickupAmount(subscriber.company_id, subscriber.cargo_id, industry_id, true);
							if (industrytransported > 0) {
								company.AddPickupIndustryCargo(subscriber.cargo_id, industrytransported, elteltMonths, industry_id);
								
								// towntransportedet el kell kuldeni
								// elvileg itt azt kuldjuk el, aki konkretan egy varos beerkezesere iratkozott fel
								foreach (asubscriber in this.subscribers) {
									local monitortypeok = false;
									if (asubscriber.monitor_type == FGCargoMonitor.MONITOR_ALL ||
										asubscriber.monitor_type == FGCargoMonitor.MONITOR_ALL_PICKUP ||
										(asubscriber.monitor_type == FGCargoMonitor.MONITOR_INDUSTRY_PICKUP && asubscriber.type_id == industry_id) ||
										(asubscriber.monitor_type == FGCargoMonitor.MONITOR_TOWN_PICKUP && asubscriber.type_id == subscriber.type_id))
										monitortypeok = true;
									
									if (monitortypeok &&
										asubscriber.company_id == company.company_id &&
										asubscriber.cargo_id == subscriber.cargo_id &&
										asubscriber.refresh_interval == FGCargoMonitor.REFRESH_IMMEDIATELY) {
										this.monitor_delegate.SendCargoUpdate(asubscriber.company_id, asubscriber.goal_id, industrytransported);
									}
								}
							}
							
							alredyMonitoredPickupIndustry.append(subscriber.cargo_id);
						}
					}
				}
			}
		}
		
		// itt a masik kereses, itt mar csak az egyeni feliratkozok az erdekesek
		foreach (subscriber in this.subscribers) {
			if (company.company_id != subscriber.company_id)
				continue;
			local alredyMonitored = null;
			
			switch (subscriber.monitor_type) {
				case FGCargoMonitor.MONITOR_TOWN_DELIVERY:
					alredyMonitored = alredyMonitoredDeliveryTown;
					break;
				case FGCargoMonitor.MONITOR_TOWN_PICKUP:
					alredyMonitored = alredyMonitoredPickupTown;
					break;
				case FGCargoMonitor.MONITOR_INDUSTRY_DELIVERY:
					alredyMonitored = alredyMonitoredDeliveryIndustry;
					break;
				case FGCargoMonitor.MONITOR_INDUSTRY_PICKUP:
					alredyMonitored = alredyMonitoredPickupIndustry;
					break;
			}
			
			if (alredyMonitored == null)
				continue;
			
			// megkeressuk, hogy nincs-e mar a monitorozott dolgok kozott
			local van = false;
			foreach (cargo_id in alredyMonitored) {
				if (cargo_id == subscriber.cargo_id) {
					van = true;
					break;
				}
			}
			
			if (van) // ha van, akkor mar elvileg kapott mindenki ertesitest
				continue;
			
			local hibavan = false;
			switch (subscriber.monitor_type) {
				case FGCargoMonitor.MONITOR_TOWN_DELIVERY:
				case FGCargoMonitor.MONITOR_TOWN_PICKUP:
					if (!GSTown.IsValidTown(subscriber.type_id)) {
						if (this.logLevel >= 2)
							GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::Update] Invalid town_id: " + subscriber.type_id);
						hibavan = true;
					}
					break;
				case FGCargoMonitor.MONITOR_INDUSTRY_DELIVERY:
				case FGCargoMonitor.MONITOR_INDUSTRY_PICKUP:
					if (!GSIndustry.IsValidIndustry(subscriber.type_id)) {
						if (this.logLevel >= 2)
							GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::Update] Invalid industry_id: " + subscriber.type_id);
						hibavan = true;
					}
					break;
			}
			
			if (hibavan)
				continue;
			
			// muszaj kulon, mert tudnunk kell, hogy valos-e az id
			local transported = 0;
			switch (subscriber.monitor_type) {
				case FGCargoMonitor.MONITOR_TOWN_DELIVERY:
					transported = GSCargoMonitor.GetTownDeliveryAmount(company.company_id, subscriber.cargo_id, subscriber.type_id, true);
					break;
				case FGCargoMonitor.MONITOR_TOWN_PICKUP:
					transported = GSCargoMonitor.GetTownPickupAmount(company.company_id, subscriber.cargo_id, subscriber.type_id, true);
					
					// TODO: ezt is ki kell venni, ha megoldjak a townpickupot industry nelkul (mint fentebb)
					// ebben az esetben az osszes varoshoz tartozo gyarat le kell ellenorizni, hogy nem szallitottak oda cuccot
					{
						local van = false;
						foreach (cargo_id in alredyMonitoredPickupIndustry) {
							if (cargo_id == subscriber.cargo_id) {
								van = true;
								break;
							}
						}
						
						if (!van) {
							// osszes varos osszes delivery-t vesszuk
							local industries = GSIndustryList_CargoProducing(subscriber.cargo_id);
							foreach (industry_id, _ in industries) {
								if (!GSIndustry.IsValidIndustry(industry_id)) {
									if (this.logLevel >= 2)
										GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::Update] (Pickup All Industry) Invalid industry id: " + industry_id);
									continue;
								}
								
								// ha nem egyezik a varos azzal, amit le akarunk kerdezni, akkor nem erdekel minket
								local a_town_id = GSTile.GetTownAuthority(GSIndustry.GetLocation(industry_id));
								if (a_town_id != subscriber.type_id)
									continue;
								
								local industrytransported = GSCargoMonitor.GetIndustryPickupAmount(subscriber.company_id, subscriber.cargo_id, industry_id, true);
								if (industrytransported > 0) {
									company.AddPickupIndustryCargo(subscriber.cargo_id, industrytransported, elteltMonths, industry_id);
									
									// towntransportedet el kell kuldeni
									// elvileg itt azt kuldjuk el, aki konkretan egy varos beerkezesere iratkozott fel
									foreach (asubscriber in this.subscribers) {
										local monitortypeok = false;
										if (asubscriber.monitor_type == FGCargoMonitor.MONITOR_ALL ||
											asubscriber.monitor_type == FGCargoMonitor.MONITOR_ALL_PICKUP ||
											(asubscriber.monitor_type == FGCargoMonitor.MONITOR_INDUSTRY_PICKUP && asubscriber.type_id == industry_id) ||
											(asubscriber.monitor_type == FGCargoMonitor.MONITOR_TOWN_PICKUP && asubscriber.type_id == subscriber.type_id))
											monitortypeok = true;
										
										if (monitortypeok &&
											asubscriber.company_id == company.company_id &&
											asubscriber.cargo_id == subscriber.cargo_id &&
											asubscriber.refresh_interval == FGCargoMonitor.REFRESH_IMMEDIATELY) {
											this.monitor_delegate.SendCargoUpdate(asubscriber.company_id, asubscriber.goal_id, industrytransported);
										}
									}
								}
								
								alredyMonitoredPickupIndustry.append(subscriber.cargo_id);
							}
						}
					}
					
					break;
				case FGCargoMonitor.MONITOR_INDUSTRY_DELIVERY:
					transported = GSCargoMonitor.GetIndustryDeliveryAmount(company.company_id, subscriber.cargo_id, subscriber.type_id, true);
					break;
				case FGCargoMonitor.MONITOR_INDUSTRY_PICKUP:
					transported = GSCargoMonitor.GetIndustryPickupAmount(company.company_id, subscriber.cargo_id, subscriber.type_id, true);
					break;
			}
			
			if (transported > 0) {
				// termeszetesen a ceg altal elszallitott cuccokhoz is bekerul
				switch (subscriber.monitor_type) {
					case FGCargoMonitor.MONITOR_TOWN_DELIVERY:
						company.AddDeliveryTownCargo(subscriber.cargo_id, transported, elteltMonths, subscriber.type_id);
						break;
					case FGCargoMonitor.MONITOR_TOWN_PICKUP:
						company.AddPickupTownCargo(subscriber.cargo_id, transported, elteltMonths, subscriber.type_id);
						break;
					case FGCargoMonitor.MONITOR_INDUSTRY_DELIVERY:
						company.AddDeliveryIndustryCargo(subscriber.cargo_id, transported, elteltMonths, subscriber.type_id);
						break;
					case FGCargoMonitor.MONITOR_INDUSTRY_PICKUP:
						company.AddPickupIndustryCargo(subscriber.cargo_id, transported, elteltMonths, subscriber.type_id);
						break;
				}
				
				// csak azokat frissitjuk, aki nem all-ra iratkozott fel, hanem csak kifejezetten town delivery-re
				foreach (asubscriber in this.subscribers) {
//					if (asubscriber.company_id == subscriber.company_id && asubscriber.cargo_id == subscriber.cargo_id && asubscriber.monitor_type == FGCargoMonitor.MONITOR_TOWN_DELIVERY && asubscriber.refresh_interval == FGCargoMonitor.REFRESH_IMMEDIATELY)
					if (asubscriber.company_id == subscriber.company_id && asubscriber.cargo_id == subscriber.cargo_id && asubscriber.refresh_interval == FGCargoMonitor.REFRESH_IMMEDIATELY)
						this.monitor_delegate.SendCargoUpdate(asubscriber.company_id, asubscriber.goal_id, transported);
				}
			}
			
			alredyMonitored.append(subscriber.cargo_id);
		}
	}
	
	// elkuldjuk a feliratkozoknak az elozo havi eredmenyeket
	if (GSDate.GetDayOfMonth(GSDate.GetCurrentDate()) == 1) {
		foreach (subscriber in this.subscribers) {
			local rm = 0;
			switch (subscriber.refresh_interval) {
				case FGCargoMonitor.REFRESH_INVALID:
				case FGCargoMonitor.REFRESH_IMMEDIATELY:
					continue;
					break;
				case FGCargoMonitor.REFRESH_EVERY_MONTHS:
					rm = 1;
					break;
				case FGCargoMonitor.REFRESH_EVERY_QARTER_YEARS:
					rm = 3;
					break;
				case FGCargoMonitor.REFRESH_EVERY_HALF_YEARS:
					rm = 6;
					break;
				case FGCargoMonitor.REFRESH_EVERY_YEARS:
					rm = 12;
					break;
			}
			
			if (rm <= 0) // bar a kisebb nem lehet...
				continue;
			
			local currentmonth = GSDate.GetMonth(GSDate.GetCurrentDate());
			currentmonth--; // akkor nem mondjuk a 3 honapban vegen kell frissiteni, hanem a 4 honap elejen, igy akkor lesz jo, ha atlapunk az uj honapba es elso napon nezzuk
			
			if ((currentmonth % rm) == 0) { // tehat akkor a frissitendo honap utolso napjan vagyunk, kuldhetjuk az update-t
				this.need_before_month = true; // at kell allitani a lekerdezes idejere ezt, hogy a helyes adatot kerdezzuk le
				local transported = this.AmountOfLastMonthGoal(subscriber.company_id, subscriber.goal_id, rm);
				this.need_before_month = false;
				if (transported > 0)
					this.monitor_delegate.SendCargoUpdate(subscriber.company_id, subscriber.goal_id, transported);
			}
		}
	}
	
	// feliratkoztatjuk a cached-ben levo varakozokat
	if (cached_subscribers.len() > 0) {
		foreach (subscriber in this.cached_subscribers) {
			this.subscribers.append(subscriber);
		}
		
		cached_subscribers.clear();
	}
}

function FGCargoMonitor::SubscribeToMonitor(company_id, cargo_id, goal_id, refresh_interval, monitor_type, type_id) {
	if (company_id == null || GSCompany.ResolveCompanyID(company_id) == GSCompany.COMPANY_INVALID) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::SubscribeToMonitor] Can not subscribe, because invalid company_id");
		return false;
	}
	
	if (cargo_id == null || !GSCargo.IsValidCargo(cargo_id)) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::SubscribeToMonitor] Can not subscribe, because invalid cargo_id");
		return false;
	}
	
	if (goal_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::SubscribeToMonitor] Can not subscribe to cargo: " + GSCargo.GetCargoLabel(cargo_id) + ", because invalid goal_id");
		return false;
	}
	
	if (!FGCargoMonitor.IsValidRefreshInterval(refresh_interval)) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::SubscribeToMonitor] Can not subscribe to cargo: " + GSCargo.GetCargoLabel(cargo_id) + ", because invalid refresh_interval");
		return false;
	}
	
	if ((monitor_type == FGCargoMonitor.MONITOR_TOWN_DELIVERY || monitor_type == FGCargoMonitor.MONITOR_TOWN_PICKUP) && type_id != null && !GSTown.IsValidTown(type_id)) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::SubscribeToMonitor] Can not subscribe to cargo: " + GSCargo.GetCargoLabel(cargo_id) + ", because invalid town_id (type_id)");
		return false;
	}
	
	if ((monitor_type == FGCargoMonitor.MONITOR_INDUSTRY_DELIVERY || monitor_type == FGCargoMonitor.MONITOR_INDUSTRY_PICKUP) && type_id != null && !GSIndustry.IsValidIndustry(type_id)) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::SubscribeToMonitor] Can not subscribe to cargo: " + GSCargo.GetCargoLabel(cargo_id) + ", because invalid industry_id (type_id)");
		return false;
	}
	
	// azert nem csinalok itt mar rogton egy feliratkozast, mert ugyis naponta frissitem, es akkor megtortenik.
	// ezaltal az van, hogy ha egyvalakinek van csak abbol az adott cargo-bol feladata, elvegzi, torlodik a monitorozas, majd megint megkapja az az adatot, akkor legalabb torlodott az elozo eredmenye.
	
	local data = {
		company_id = company_id,
		cargo_id = cargo_id,
		goal_id = goal_id,
		refresh_interval = refresh_interval,
		monitor_type = monitor_type,
		type_id = type_id,
	}
	
	local monitored_goal = FGMonitoredGoal(data);
	this.cached_subscribers.append(monitored_goal);
	
	if (this.logLevel >= 3)
		GSLog.Info(this.GetLogGameDateString() + "[FGCargoMonitor::SubscribeToMonitor] " + GSCompany.GetName(company_id) + " succesfull subscribed to cargo: " + GSCargo.GetCargoLabel(cargo_id) + ", goal_id: " + goal_id);
	
	return true;
}

function FGCargoMonitor::UnSubscribeFromMonitor(company_id, goal_id) {
	if (company_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::UnSubscribeFromMonitor] Can not unsubscribe, because invalid company_id");
		return false;
	}
	
	local compname = GSCompany.GetName(company_id);
	if (compname == null || compname == "")
		compname = "(Bankrupted or Merged Company)";
	
	if (this.logLevel >= 3)
		GSLog.Info(this.GetLogGameDateString() + "[FGCargoMonitor::UnSubscribeFromMonitor] " + compname + " begin unsubscribe from CargoMonitor, goal_id: " + goal_id);
	
	
	local subscriber = null;
	for (local i = 0; i < this.subscribers.len(); i++) {
		local asubscriber = this.subscribers[i];
		if (asubscriber.company_id == company_id && asubscriber.goal_id == goal_id) {
			subscriber = asubscriber;
			this.subscribers.remove(i);
			if (this.logLevel >= 3)
				GSLog.Info(this.GetLogGameDateString() + "[FGCargoMonitor::UnSubscribeFromMonitor] " + compname + " have monitored cargo: " + GSCargo.GetCargoLabel(asubscriber.cargo_id));
			break;
		}
	}
	
	if (subscriber == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::UnSubscribeFromMonitor] " + compname + " not have monitored goal_id: " + goal_id);
		return;
	}
	
	local company = this.CompanyFromID(company_id);
	if (company == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::UnSubscribeFromMonitor] " + compname + " not have valid tracking object");
		return;
	}
	
	company.Remove(subscriber, this.subscribers);
}

function FGCargoMonitor::UnSubscribeCompanyFromMonitor(company_id) {
	if (company_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::UnSubscribeCompanyFromMonitor] Can not unsubscribe, because invalid company_id");
		return false;
	}
	
	for (local i = 0; i < this.companies.len(); i++) {
		if (this.companies[i].company_id == company_id) {
			local company = this.companies[i];
			for (local j = this.subscribers.len() - 1; j >= 0; j--) { // igy nem lesz gond az eltavolitas miatt...
				local subscriber = this.subscribers[j];
				if (company.company_id == subscriber.company_id) {
					this.UnSubscribeFromMonitor(company.company_id, subscriber.goal_id);
					break;

					// az unsubscribernel eltavolitom, ezert nem kell itt.
//					this.subscribers.remove(j);
//					j--;
				}
			}
			
			this.companies.remove(i);
			break;
		}
	}
}

function FGCargoMonitor::UnSubscribeAllFromMonitor() {
	// azert nem hasznaljuk a FGCargoMonitor::UnSubscribeCompanyFromMonitor funkciot, mert meg egyszer tok feleslegesen lefuttatna az osszes companies-t
	foreach (subscriber in this.subscribers) {
		this.UnSubscribeFromMonitor(subscriber.company_id, subscriber.goal_id);
	}
	
	this.companies.clear();
	this.subscribers.clear();
	this.cached_subscribers.clear();
	GSCargoMonitor.StopAllMonitoring();
}

function FGCargoMonitor::IsValidRefreshInterval(refresh_interval) {
	if (refresh_interval == null)
		return false;
	
	switch (refresh_interval) {
		case FGCargoMonitor.REFRESH_IMMEDIATELY:
		case FGCargoMonitor.REFRESH_EVERY_MONTHS:
		case FGCargoMonitor.REFRESH_EVERY_QARTER_YEARS:
		case FGCargoMonitor.REFRESH_EVERY_HALF_YEARS:
		case FGCargoMonitor.REFRESH_EVERY_YEARS:
			return true;
	}
	
	return false;
}



// marmint ez az osszeset jelenti a delivery and pickupot egyben
/* ezt kiveszem, mert hulyeseg...
function FGCargoMonitor::AmountOfLastMonthTransportedCargo(company_id, cargo_id, last_months) {
	if (company_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfLastMonthTransportedCargo] Can not query, because invalid company_id");
		return 0;
	}
	
	if (cargo_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfLastMonthTransportedCargo] Can not query, because invalid cargo_id");
		return 0;
	}
	
	if (last_months <= 0)
		return this.AmountOfAllTransportedCargo(company_id, cargo_id);
	
	// azert + 1, mert a kovetkezo honap elsejen (es a nap elejen) nezzuk, es akkor meg nincs semmi sem benne
	if (this.need_before_month)
		last_months++;
	
	foreach (company in this.companies) {
		if (company.company_id == company_id) {
			local transported = 0;
			
			local delivery_cargo_object = company.CargoObject(cargo_id, company.cargos_delivery);
			if (delivery_cargo_object != null) {
				
				local from = delivery_cargo_object.months.len() - last_months;
				if (from < 0)
					from = 0;
				
				for (local i = from; i < delivery_cargo_object.months.len(); i++)
					transported += delivery_cargo_object.months[i];
				
			}
			
			local pickup_cargo_object = company.CargoObject(cargo_id, company.cargos_pickup);
			if (pickup_cargo_object != null) {
				
				local from = pickup_cargo_object.months.len() - last_months;
				if (from < 0)
					from = 0;
				
				for (local i = from; i < pickup_cargo_object.months.len(); i++)
					transported += pickup_cargo_object.months[i];
				
			}
			
			return transported;
			break;
		}
	}
	
	return 0;
}

function FGCargoMonitor::AmountOfAllTransportedCargo(company_id, cargo_id) {
	if (company_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfAllTransportedCargo] Can not query, because invalid company_id");
		return 0;
	}
	
	if (cargo_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfAllTransportedCargo] Can not query, because invalid cargo_id");
		return 0;
	}
	
	foreach (company in this.companies) {
		if (company.company_id == company_id) {
			local transported = 0;
			
			local delivery_cargo_object = company.CargoObject(cargo_id, company.cargos_delivery);
			if (delivery_cargo_object != null) {
				for (local i = 0; i < delivery_cargo_object.months.len(); i++)
					transported += delivery_cargo_object.months[i];
			}
			
			local pickup_cargo_object = company.CargoObject(cargo_id, company.cargos_pickup);
			if (pickup_cargo_object != null) {
				for (local i = 0; i < pickup_cargo_object.months.len(); i++)
					transported += pickup_cargo_object.months[i];
			}
			
			return transported;
			
			break;
		}
	}
	
	return 0;
}
*/

function FGCargoMonitor::AmountOfLastMonthDeliveredCargo(company_id, cargo_id, last_months) {
	if (company_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfLastMonthDeliveredCargo] Can not query, because invalid company_id");
		return 0;
	}
	
	if (cargo_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfLastMonthDeliveredCargo] Can not query, because invalid cargo_id");
		return 0;
	}
	
	if (last_months <= 0)
		return this.AmountOfAllTransportedCargo(company_id, cargo_id);
	
	// azert + 1, mert a kovetkezo honap elsejen (es a nap elejen) nezzuk, es akkor meg nincs semmi sem benne
	if (this.need_before_month)
		last_months++;
	
	foreach (company in this.companies) {
		if (company.company_id == company_id) {
			local transported = 0;
			
			local delivery_cargo_object = company.CargoObject(cargo_id, company.cargos_delivery);
			if (delivery_cargo_object != null) {
				
				local from = delivery_cargo_object.months.len() - last_months;
				if (from < 0)
					from = 0;
				
				for (local i = from; i < delivery_cargo_object.months.len(); i++)
					transported += delivery_cargo_object.months[i];
				
			}
			
			return transported;
			break;
		}
	}
	
	return 0;
}

function FGCargoMonitor::AmountOfAllDeliveredCargo(company_id, cargo_id) {
	if (company_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfAllDeliveredCargo] Can not query, because invalid company_id");
		return 0;
	}
	
	if (cargo_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfAllDeliveredCargo] Can not query, because invalid cargo_id");
		return 0;
	}
	
	foreach (company in this.companies) {
		if (company.company_id == company_id) {
			local transported = 0;
			
			local delivery_cargo_object = company.CargoObject(cargo_id, company.cargos_delivery);
			if (delivery_cargo_object != null) {
				for (local i = 0; i < delivery_cargo_object.months.len(); i++)
					transported += delivery_cargo_object.months[i];
			}
			
			return transported;
			
			break;
		}
	}
	
	return 0;
}

function FGCargoMonitor::AmountOfLastMonthPickedupCargo(company_id, cargo_id, last_months) {
	if (company_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfLastMonthPickedupCargo] Can not query, because invalid company_id");
		return 0;
	}
	
	if (cargo_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfLastMonthPickedupCargo] Can not query, because invalid cargo_id");
		return 0;
	}
	
	if (last_months <= 0)
		return this.AmountOfAllTransportedCargo(company_id, cargo_id);
	
	// azert + 1, mert a kovetkezo honap elsejen (es a nap elejen) nezzuk, es akkor meg nincs semmi sem benne
	if (this.need_before_month)
		last_months++;
	
	foreach (company in this.companies) {
		if (company.company_id == company_id) {
			local transported = 0;
			
			local pickup_cargo_object = company.CargoObject(cargo_id, company.cargos_pickup);
			if (pickup_cargo_object != null) {
				
				local from = pickup_cargo_object.months.len() - last_months;
				if (from < 0)
					from = 0;
				
				for (local i = from; i < pickup_cargo_object.months.len(); i++)
					transported += pickup_cargo_object.months[i];
				
			}
			
			return transported;
			break;
		}
	}
	
	return 0;
}

function FGCargoMonitor::AmountOfAllPickedupCargo(company_id, cargo_id) {
	if (company_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfAllPickedupCargo] Can not query, because invalid company_id");
		return 0;
	}
	
	if (cargo_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfAllPickedupCargo] Can not query, because invalid cargo_id");
		return 0;
	}
	
	foreach (company in this.companies) {
		if (company.company_id == company_id) {
			local transported = 0;
			
			local pickup_cargo_object = company.CargoObject(cargo_id, company.cargos_pickup);
			if (pickup_cargo_object != null) {
				for (local i = 0; i < pickup_cargo_object.months.len(); i++)
					transported += pickup_cargo_object.months[i];
			}
			
			return transported;
			
			break;
		}
	}
	
	return 0;
}

function FGCargoMonitor::AmountOfLastMonthGoal(company_id, goal_id, last_months) {
	if (company_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfLastMonthGoal] Can not query, because invalid company_id");
		return 0;
	}
	
	if (goal_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfLastMonthGoal] Can not query, because invalid goal_id");
		return 0;
	}
	
	if (last_months <= 0)
		return this.AmountOfAllGoal(company_id, goal_id);
	
	local subscriber = null;
	foreach (asubscriber in this.subscribers) {
		if (asubscriber.company_id == company_id && asubscriber.goal_id == goal_id) {
			subscriber = asubscriber;
			break;
		}
	}
	
	if (subscriber == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfLastMonthGoal] Invalid goal_id or company_id");
		return 0;
	}
	
	local company = null;
	foreach (acompany in this.companies) {
		if (acompany.company_id == company_id) {
			company = acompany;
			break;
		}
	}
	
	if (company == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfLastMonthGoal] Invalid company_id");
		return 0;
	}
	
	local cargo_object_list = null;
	
	switch (subscriber.monitor_type) {
		case FGCargoMonitor.MONITOR_ALL:
			cargo_object_list = company.cargos_delivery;
			break;
		case FGCargoMonitor.MONITOR_ALL_DELIVERY:
			cargo_object_list = company.cargos_delivery;
			break;
		case FGCargoMonitor.MONITOR_ALL_PICKUP:
			cargo_object_list = company.cargos_pickup;
			break;
		case FGCargoMonitor.MONITOR_TOWN_DELIVERY:
			local town_object = company.TownObject(subscriber.type_id, company.town_delivery);
			if (town_object != null)
				cargo_object_list = town_object.cargos_delivery;
			break;
		case FGCargoMonitor.MONITOR_TOWN_PICKUP:
			local town_object = company.TownObject(subscriber.type_id, company.town_pickup);
			if (town_object != null)
				cargo_object_list = town_object.cargos_pickup;
			break;
		case FGCargoMonitor.MONITOR_INDUSTRY_DELIVERY:
			local industry_object = company.IndustryObject(subscriber.type_id, company.industry_delivery);
			if (industry_object != null)
				cargo_object_list = industry_object.cargos_delivery;
			break;
		case FGCargoMonitor.MONITOR_INDUSTRY_PICKUP:
			local industry_object = company.IndustryObject(subscriber.type_id, company.industry_pickup);
			if (industry_object != null)
				cargo_object_list = industry_object.cargos_pickup;
			break;
	}
	
	if (cargo_object_list == null) {
		// ez nem hiba, mert elofordul, hog feliratkoztunk valamire, de meg nem erkezett be semmi, ezert nincs ilyen lista
//		if (this.logLevel >= 2)
//			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfLastMonthGoal] Invalid cargo_object_list");
		return 0;
	}
	
	local cargo_object = null;
	foreach (acargo_object in cargo_object_list) {
		if (acargo_object.cargo_id == subscriber.cargo_id)
			cargo_object = acargo_object;
	}
	
	if (cargo_object == null) {
		// ez sem hiba, mert ha nem azonnal ertesulunk, akkor elofordul olyan, hogy meg nem vegeztunk ilyen feladatot, azert nincs errol bejegyzes
//		if (this.logLevel >= 2)
//			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfLastMonthGoal] Invalid cargo_object");
		return 0;
	}
	
	local transported = 0;
	
	// azert + 1, mert a kovetkezo honap elsejen (es a nap elejen) nezzuk, es akkor meg nincs semmi sem benne
	if (this.need_before_month)
		last_months++;
	
	local from = cargo_object.months.len() - last_months;
	if (from < 0)
		from = 0;
	
	
	for (local i = from; i < cargo_object.months.len(); i++)
		transported += cargo_object.months[i];
	
	// azert kell ez, mert ha all, akkor mindket tipusra le kell futtatni....
	if (subscriber.monitor_type == FGCargoMonitor.MONITOR_ALL) {
		cargo_object = company.cargos_pickup;
		
		local from = cargo_object.months.len() - last_months;
		if (from < 0)
			from = 0;
		
		for (local i = from; i < cargo_object.months.len(); i++)
			transported += cargo_object.months[i];
	}
	
	return transported;
}

function FGCargoMonitor::AmountOfAllGoal(company_id, goal_id) {
	if (company_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfAllGoal] Can not query, because invalid company_id");
		return 0;
	}
	
	if (goal_id == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfAllGoal] Can not query, because invalid goal_id");
		return 0;
	}
	
	local subscriber = null;
	foreach (asubscriber in this.subscribers) {
		if (asubscriber.company_id == company_id && asubscriber.goal_id == goal_id)
			subscriber = asubscriber;
	}
	
	if (subscriber == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfAllGoal] Invalid goal_id or company_id");
		return 0;
	}
	
	local company = null;
	foreach (acompany in this.companies) {
		if (acompany.company_id == company_id) {
			company = acompany;
			break;
		}
	}
	
	if (company == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfAllGoal] Invalid company_id");
		return 0;
	}
	
	local cargo_object_list = null;
	switch (subscriber.monitor_type) {
		case FGCargoMonitor.MONITOR_ALL:
			cargo_object_list = company.cargos_delivery;
			break;
		case FGCargoMonitor.MONITOR_ALL_DELIVERY:
			cargo_object_list = company.cargos_delivery;
			break;
		case FGCargoMonitor.MONITOR_ALL_PICKUP:
			cargo_object_list = company.cargos_pickup;
			break;
		case FGCargoMonitor.MONITOR_TOWN_DELIVERY:
			local town_object = company.TownObject(subscriber.type_id, company.town_delivery);
			if (town_object != null)
				cargo_object_list = town_object.cargos_delivery;
			break;
		case FGCargoMonitor.MONITOR_TOWN_PICKUP:
			local town_object = company.TownObject(subscriber.type_id, company.town_pickup);
			if (town_object != null)
				cargo_object_list = town_object.cargos_pickup;
			break;
		case FGCargoMonitor.MONITOR_INDUSTRY_DELIVERY:
			local industry_object = company.IndustryObject(subscriber.type_id, company.industry_delivery);
			if (industry_object != null)
				cargo_object_list = industry_object.cargos_delivery;
			break;
		case FGCargoMonitor.MONITOR_INDUSTRY_PICKUP:
			local industry_object = company.IndustryObject(subscriber.type_id, company.industry_pickup);
			if (industry_object != null)
				cargo_object_list = industry_object.cargos_pickup;
			break;
	}
	
	if (cargo_object_list == null) {
		// ez nem hiba, mert elofordul, hog feliratkoztunk valamire, de meg nem erkezett be semmi, ezert nincs ilyen lista
//		if (this.logLevel >= 2)
//			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfLastMonthGoal] Invalid cargo_object_list");
		return 0;
	}
	
	
	local cargo_object = null;
	foreach (acargo_object in cargo_object_list) {
		if (acargo_object.cargo_id == subscriber.cargo_id)
			cargo_object = acargo_object;
	}
	
	if (cargo_object == null) {
		if (this.logLevel >= 2)
			GSLog.Warning(this.GetLogGameDateString() + "[FGCargoMonitor::AmountOfAllGoal] Invalid cargo_object");
		return 0;
	}
	
	local transported = 0;
	
	for (local i = 0; i < cargo_object.months.len(); i++)
		transported += cargo_object.months[i];
	
	if (subscriber.monitor_type == FGCargoMonitor.MONITOR_ALL) {
		cargo_object = company.cargos_pickup;
		
		for (local i = 0; i < cargo_object.months.len(); i++)
			transported += cargo_object.months[i];
	}
	
	return transported;
}

//
// Private functions
//

function FGCargoMonitor::CompanyFromID(company_id) {
	foreach (company in this.companies) {
		if (company.company_id == company_id)
			return company;
	}
	
	return null;
}
